# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.browser_rendering.devtools import session_list_params
from ....types.browser_rendering.devtools.session_get_response import SessionGetResponse
from ....types.browser_rendering.devtools.session_list_response import SessionListResponse

__all__ = ["SessionResource", "AsyncSessionResource"]


class SessionResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SessionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SessionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SessionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SessionResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        limit: float | Omit = omit,
        offset: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionListResponse:
        """
        List active browser sessions.

        Args:
          account_id: Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/browser-rendering/devtools/session", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    session_list_params.SessionListParams,
                ),
            ),
            cast_to=SessionListResponse,
        )

    def get(
        self,
        session_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SessionGetResponse]:
        """
        Get details for a specific browser session.

        Args:
          account_id: Account ID.

          session_id: Session ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/session/{session_id}",
                account_id=account_id,
                session_id=session_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionGetResponse,
        )


class AsyncSessionResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSessionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSessionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSessionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSessionResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        limit: float | Omit = omit,
        offset: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionListResponse:
        """
        List active browser sessions.

        Args:
          account_id: Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/browser-rendering/devtools/session", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    session_list_params.SessionListParams,
                ),
            ),
            cast_to=SessionListResponse,
        )

    async def get(
        self,
        session_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SessionGetResponse]:
        """
        Get details for a specific browser session.

        Args:
          account_id: Account ID.

          session_id: Session ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/session/{session_id}",
                account_id=account_id,
                session_id=session_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionGetResponse,
        )


class SessionResourceWithRawResponse:
    def __init__(self, session: SessionResource) -> None:
        self._session = session

        self.list = to_raw_response_wrapper(
            session.list,
        )
        self.get = to_raw_response_wrapper(
            session.get,
        )


class AsyncSessionResourceWithRawResponse:
    def __init__(self, session: AsyncSessionResource) -> None:
        self._session = session

        self.list = async_to_raw_response_wrapper(
            session.list,
        )
        self.get = async_to_raw_response_wrapper(
            session.get,
        )


class SessionResourceWithStreamingResponse:
    def __init__(self, session: SessionResource) -> None:
        self._session = session

        self.list = to_streamed_response_wrapper(
            session.list,
        )
        self.get = to_streamed_response_wrapper(
            session.get,
        )


class AsyncSessionResourceWithStreamingResponse:
    def __init__(self, session: AsyncSessionResource) -> None:
        self._session = session

        self.list = async_to_streamed_response_wrapper(
            session.list,
        )
        self.get = async_to_streamed_response_wrapper(
            session.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["BrowserIsolationSettingsParam"]


class BrowserIsolationSettingsParam(TypedDict, total=False):
    """Specify Clientless Browser Isolation settings."""

    non_identity_enabled: bool
    """Specify whether to enable non-identity onramp support for Browser Isolation."""

    url_browser_isolation_enabled: bool
    """Specify whether to enable Clientless Browser Isolation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ...._utils import PropertyInfo
from ...._models import BaseModel
from .consumer_script import ConsumerScript
from ..single_step_migration import SingleStepMigration

__all__ = [
    "ScriptAndVersionSettingGetResponse",
    "Annotations",
    "Binding",
    "BindingWorkersBindingKindAI",
    "BindingWorkersBindingKindAISearch",
    "BindingWorkersBindingKindAISearchNamespace",
    "BindingWorkersBindingKindAnalyticsEngine",
    "BindingWorkersBindingKindAssets",
    "BindingWorkersBindingKindBrowser",
    "BindingWorkersBindingKindD1",
    "BindingWorkersBindingKindDataBlob",
    "BindingWorkersBindingKindDispatchNamespace",
    "BindingWorkersBindingKindDispatchNamespaceOutbound",
    "BindingWorkersBindingKindDispatchNamespaceOutboundParam",
    "BindingWorkersBindingKindDispatchNamespaceOutboundWorker",
    "BindingWorkersBindingKindDurableObjectNamespace",
    "BindingWorkersBindingKindHyperdrive",
    "BindingWorkersBindingKindInherit",
    "BindingWorkersBindingKindImages",
    "BindingWorkersBindingKindJson",
    "BindingWorkersBindingKindKVNamespace",
    "BindingWorkersBindingKindMedia",
    "BindingWorkersBindingKindMTLSCertificate",
    "BindingWorkersBindingKindPlainText",
    "BindingWorkersBindingKindPipelines",
    "BindingWorkersBindingKindQueue",
    "BindingWorkersBindingKindRatelimit",
    "BindingWorkersBindingKindRatelimitSimple",
    "BindingWorkersBindingKindR2Bucket",
    "BindingWorkersBindingKindSecretText",
    "BindingWorkersBindingKindSendEmail",
    "BindingWorkersBindingKindService",
    "BindingWorkersBindingKindTextBlob",
    "BindingWorkersBindingKindVectorize",
    "BindingWorkersBindingKindVersionMetadata",
    "BindingWorkersBindingKindSecretsStoreSecret",
    "BindingWorkersBindingKindFlagship",
    "BindingWorkersBindingKindSecretKey",
    "BindingWorkersBindingKindWorkflow",
    "BindingWorkersBindingKindWasmModule",
    "BindingWorkersBindingKindVPCService",
    "BindingWorkersBindingKindVPCNetwork",
    "Limits",
    "Migrations",
    "MigrationsWorkersMultipleStepMigrations",
    "Observability",
    "ObservabilityLogs",
    "ObservabilityTraces",
    "Placement",
    "PlacementMode",
    "PlacementRegion",
    "PlacementHostname",
    "PlacementHost",
    "PlacementUnionMember4",
    "PlacementUnionMember5",
    "PlacementUnionMember6",
    "PlacementUnionMember7",
    "PlacementUnionMember7Target",
    "PlacementUnionMember7TargetRegion",
    "PlacementUnionMember7TargetHostname",
    "PlacementUnionMember7TargetHost",
]


class Annotations(BaseModel):
    """Annotations for the Worker version.

    Annotations are not inherited across settings updates; omitting this field means the new version will have no annotations.
    """

    workers_message: Optional[str] = FieldInfo(alias="workers/message", default=None)
    """Human-readable message about the version. Truncated to 1000 bytes if longer."""

    workers_tag: Optional[str] = FieldInfo(alias="workers/tag", default=None)
    """User-provided identifier for the version. Maximum 100 bytes."""

    workers_triggered_by: Optional[str] = FieldInfo(alias="workers/triggered_by", default=None)
    """Operation that triggered the creation of the version.

    This is read-only and set by the server.
    """


class BindingWorkersBindingKindAI(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["ai"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindAISearch(BaseModel):
    instance_name: str
    """The user-chosen instance name.

    Must exist at deploy time. The worker can search, chat, update, and manage
    items/jobs on this instance.
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["ai_search"]
    """The kind of resource that the binding provides."""

    namespace: Optional[str] = None
    """The namespace the instance belongs to.

    Defaults to "default" if omitted. Customers who don't use namespaces can simply
    omit this field.
    """


class BindingWorkersBindingKindAISearchNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace: str
    """The user-chosen namespace name.

    Must exist before deploy -- Wrangler handles auto-creation on deploy failure (R2
    bucket pattern). The "default" namespace is auto-created by config-api for new
    accounts. Grants full access (CRUD + search + chat) to all instances within the
    namespace.
    """

    type: Literal["ai_search_namespace"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindAnalyticsEngine(BaseModel):
    dataset: str
    """The name of the dataset to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["analytics_engine"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindAssets(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["assets"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindBrowser(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["browser"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindD1(BaseModel):
    database_id: str
    """Identifier of the D1 database to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["d1"]
    """The kind of resource that the binding provides."""

    id: Optional[str] = None
    """Identifier of the D1 database to bind to."""


class BindingWorkersBindingKindDataBlob(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the data content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["data_blob"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindDispatchNamespaceOutboundParam(BaseModel):
    name: str
    """Name of the parameter."""


class BindingWorkersBindingKindDispatchNamespaceOutboundWorker(BaseModel):
    """Outbound worker."""

    entrypoint: Optional[str] = None
    """Entrypoint to invoke on the outbound worker."""

    environment: Optional[str] = None
    """Environment of the outbound worker."""

    service: Optional[str] = None
    """Name of the outbound worker."""


class BindingWorkersBindingKindDispatchNamespaceOutbound(BaseModel):
    """Outbound worker."""

    params: Optional[List[BindingWorkersBindingKindDispatchNamespaceOutboundParam]] = None
    """
    Pass information from the Dispatch Worker to the Outbound Worker through the
    parameters.
    """

    worker: Optional[BindingWorkersBindingKindDispatchNamespaceOutboundWorker] = None
    """Outbound worker."""


class BindingWorkersBindingKindDispatchNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace: str
    """The name of the dispatch namespace."""

    type: Literal["dispatch_namespace"]
    """The kind of resource that the binding provides."""

    outbound: Optional[BindingWorkersBindingKindDispatchNamespaceOutbound] = None
    """Outbound worker."""


class BindingWorkersBindingKindDurableObjectNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["durable_object_namespace"]
    """The kind of resource that the binding provides."""

    class_name: Optional[str] = None
    """The exported class name of the Durable Object."""

    dispatch_namespace: Optional[str] = None
    """The dispatch namespace the Durable Object script belongs to."""

    environment: Optional[str] = None
    """The environment of the script_name to bind to."""

    namespace_id: Optional[str] = None
    """Namespace identifier tag."""

    script_name: Optional[str] = None
    """
    The script where the Durable Object is defined, if it is external to this
    Worker.
    """


class BindingWorkersBindingKindHyperdrive(BaseModel):
    id: str
    """Identifier of the Hyperdrive connection to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["hyperdrive"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindInherit(BaseModel):
    name: str
    """The name of the inherited binding."""

    type: Literal["inherit"]
    """The kind of resource that the binding provides."""

    old_name: Optional[str] = None
    """The old name of the inherited binding.

    If set, the binding will be renamed from `old_name` to `name` in the new
    version. If not set, the binding will keep the same name between versions.
    """

    version_id: Optional[str] = None
    """
    Identifier for the version to inherit the binding from, which can be the version
    ID or the literal "latest" to inherit from the latest version. Defaults to
    inheriting the binding from the latest version.
    """


class BindingWorkersBindingKindImages(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["images"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindJson(BaseModel):
    json_: object = FieldInfo(alias="json")
    """JSON data to use."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["json"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindKVNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace_id: str
    """Namespace identifier tag."""

    type: Literal["kv_namespace"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindMedia(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["media"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindMTLSCertificate(BaseModel):
    certificate_id: str
    """Identifier of the certificate to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["mtls_certificate"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindPlainText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    text: str
    """The text value to use."""

    type: Literal["plain_text"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindPipelines(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    pipeline: str
    """Name of the Pipeline to bind to."""

    type: Literal["pipelines"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindQueue(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    queue_name: str
    """Name of the Queue to bind to."""

    type: Literal["queue"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindRatelimitSimple(BaseModel):
    """The rate limit configuration."""

    limit: float
    """The limit (requests per period)."""

    period: int
    """The period in seconds."""

    mitigation_timeout: Optional[int] = None
    """
    Duration in seconds to apply the mitigation action after the rate limit is
    exceeded. Valid values are 0 (disabled), 10, or multiples of 60 up to 86400.
    Must be greater than or equal to the period when non-zero.
    """


class BindingWorkersBindingKindRatelimit(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace_id: str
    """Identifier of the rate limit namespace to bind to."""

    simple: BindingWorkersBindingKindRatelimitSimple
    """The rate limit configuration."""

    type: Literal["ratelimit"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindR2Bucket(BaseModel):
    bucket_name: str
    """R2 bucket to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["r2_bucket"]
    """The kind of resource that the binding provides."""

    jurisdiction: Optional[Literal["eu", "fedramp", "fedramp-high"]] = None
    """
    The
    [jurisdiction](https://developers.cloudflare.com/r2/reference/data-location/#jurisdictional-restrictions)
    of the R2 bucket.
    """


class BindingWorkersBindingKindSecretText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_text"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindSendEmail(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["send_email"]
    """The kind of resource that the binding provides."""

    allowed_destination_addresses: Optional[List[str]] = None
    """List of allowed destination addresses."""

    allowed_sender_addresses: Optional[List[str]] = None
    """List of allowed sender addresses."""

    destination_address: Optional[str] = None
    """Destination address for the email."""


class BindingWorkersBindingKindService(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    service: str
    """Name of Worker to bind to."""

    type: Literal["service"]
    """The kind of resource that the binding provides."""

    entrypoint: Optional[str] = None
    """Entrypoint to invoke on the target Worker."""

    environment: Optional[str] = None
    """Optional environment if the Worker utilizes one."""


class BindingWorkersBindingKindTextBlob(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the text content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["text_blob"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindVectorize(BaseModel):
    index_name: str
    """Name of the Vectorize index to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["vectorize"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindVersionMetadata(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["version_metadata"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindSecretsStoreSecret(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    secret_name: str
    """Name of the secret in the store."""

    store_id: str
    """ID of the store containing the secret."""

    type: Literal["secrets_store_secret"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindFlagship(BaseModel):
    app_id: str
    """ID of the Flagship app to bind to for feature flag evaluation."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["flagship"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindSecretKey(BaseModel):
    algorithm: object
    """Algorithm-specific key parameters.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#algorithm).
    """

    format: Literal["raw", "pkcs8", "spki", "jwk"]
    """Data format of the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#format).
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_key"]
    """The kind of resource that the binding provides."""

    usages: List[Literal["encrypt", "decrypt", "sign", "verify", "deriveKey", "deriveBits", "wrapKey", "unwrapKey"]]
    """Allowed operations with the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#keyUsages).
    """


class BindingWorkersBindingKindWorkflow(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["workflow"]
    """The kind of resource that the binding provides."""

    workflow_name: str
    """Name of the Workflow to bind to."""

    class_name: Optional[str] = None
    """Class name of the Workflow.

    Should only be provided if the Workflow belongs to this script.
    """

    script_name: Optional[str] = None
    """Script name that contains the Workflow.

    If not provided, defaults to this script name.
    """


class BindingWorkersBindingKindWasmModule(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the WebAssembly module content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["wasm_module"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindVPCService(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    service_id: str
    """Identifier of the VPC service to bind to."""

    type: Literal["vpc_service"]
    """The kind of resource that the binding provides."""


class BindingWorkersBindingKindVPCNetwork(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["vpc_network"]
    """The kind of resource that the binding provides."""

    network_id: Optional[str] = None
    """Identifier of the network to bind to.

    Only "cf1:network" is currently supported. Mutually exclusive with tunnel_id.
    """

    tunnel_id: Optional[str] = None
    """UUID of the Cloudflare Tunnel to bind to. Mutually exclusive with network_id."""


Binding: TypeAlias = Annotated[
    Union[
        BindingWorkersBindingKindAI,
        BindingWorkersBindingKindAISearch,
        BindingWorkersBindingKindAISearchNamespace,
        BindingWorkersBindingKindAnalyticsEngine,
        BindingWorkersBindingKindAssets,
        BindingWorkersBindingKindBrowser,
        BindingWorkersBindingKindD1,
        BindingWorkersBindingKindDataBlob,
        BindingWorkersBindingKindDispatchNamespace,
        BindingWorkersBindingKindDurableObjectNamespace,
        BindingWorkersBindingKindHyperdrive,
        BindingWorkersBindingKindInherit,
        BindingWorkersBindingKindImages,
        BindingWorkersBindingKindJson,
        BindingWorkersBindingKindKVNamespace,
        BindingWorkersBindingKindMedia,
        BindingWorkersBindingKindMTLSCertificate,
        BindingWorkersBindingKindPlainText,
        BindingWorkersBindingKindPipelines,
        BindingWorkersBindingKindQueue,
        BindingWorkersBindingKindRatelimit,
        BindingWorkersBindingKindR2Bucket,
        BindingWorkersBindingKindSecretText,
        BindingWorkersBindingKindSendEmail,
        BindingWorkersBindingKindService,
        BindingWorkersBindingKindTextBlob,
        BindingWorkersBindingKindVectorize,
        BindingWorkersBindingKindVersionMetadata,
        BindingWorkersBindingKindSecretsStoreSecret,
        BindingWorkersBindingKindFlagship,
        BindingWorkersBindingKindSecretKey,
        BindingWorkersBindingKindWorkflow,
        BindingWorkersBindingKindWasmModule,
        BindingWorkersBindingKindVPCService,
        BindingWorkersBindingKindVPCNetwork,
    ],
    PropertyInfo(discriminator="type"),
]


class Limits(BaseModel):
    """Limits to apply for this Worker."""

    cpu_ms: Optional[int] = None
    """The amount of CPU time this Worker can use in milliseconds."""

    subrequests: Optional[int] = None
    """The number of subrequests this Worker can make per request."""


class MigrationsWorkersMultipleStepMigrations(BaseModel):
    pass


Migrations: TypeAlias = Union[SingleStepMigration, MigrationsWorkersMultipleStepMigrations]


class ObservabilityLogs(BaseModel):
    """Log settings for the Worker."""

    enabled: bool
    """Whether logs are enabled for the Worker."""

    invocation_logs: bool
    """
    Whether
    [invocation logs](https://developers.cloudflare.com/workers/observability/logs/workers-logs/#invocation-logs)
    are enabled for the Worker.
    """

    destinations: Optional[List[str]] = None
    """A list of destinations where logs will be exported to."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for logs. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: Optional[bool] = None
    """Whether log persistence is enabled for the Worker."""


class ObservabilityTraces(BaseModel):
    """Trace settings for the Worker."""

    destinations: Optional[List[str]] = None
    """A list of destinations where traces will be exported to."""

    enabled: Optional[bool] = None
    """Whether traces are enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for traces. From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1."""

    persist: Optional[bool] = None
    """Whether trace persistence is enabled for the Worker."""


class Observability(BaseModel):
    """Observability settings for the Worker."""

    enabled: bool
    """Whether observability is enabled for the Worker."""

    head_sampling_rate: Optional[float] = None
    """The sampling rate for incoming requests.

    From 0 to 1 (1 = 100%, 0.1 = 10%). Default is 1.
    """

    logs: Optional[ObservabilityLogs] = None
    """Log settings for the Worker."""

    traces: Optional[ObservabilityTraces] = None
    """Trace settings for the Worker."""


class PlacementMode(BaseModel):
    mode: Literal["smart"]
    """
    Enables
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    """


class PlacementRegion(BaseModel):
    region: str
    """Cloud region for targeted placement in format 'provider:region'."""


class PlacementHostname(BaseModel):
    hostname: str
    """HTTP hostname for targeted placement."""


class PlacementHost(BaseModel):
    host: str
    """TCP host and port for targeted placement."""


class PlacementUnionMember4(BaseModel):
    mode: Literal["targeted"]
    """Targeted placement mode."""

    region: str
    """Cloud region for targeted placement in format 'provider:region'."""


class PlacementUnionMember5(BaseModel):
    hostname: str
    """HTTP hostname for targeted placement."""

    mode: Literal["targeted"]
    """Targeted placement mode."""


class PlacementUnionMember6(BaseModel):
    host: str
    """TCP host and port for targeted placement."""

    mode: Literal["targeted"]
    """Targeted placement mode."""


class PlacementUnionMember7TargetRegion(BaseModel):
    region: str
    """Cloud region in format 'provider:region'."""


class PlacementUnionMember7TargetHostname(BaseModel):
    hostname: str
    """HTTP hostname for targeted placement."""


class PlacementUnionMember7TargetHost(BaseModel):
    host: str
    """TCP host:port for targeted placement."""


PlacementUnionMember7Target: TypeAlias = Union[
    PlacementUnionMember7TargetRegion, PlacementUnionMember7TargetHostname, PlacementUnionMember7TargetHost
]


class PlacementUnionMember7(BaseModel):
    mode: Literal["targeted"]
    """Targeted placement mode."""

    target: List[PlacementUnionMember7Target]
    """Array of placement targets (currently limited to single target)."""


Placement: TypeAlias = Union[
    PlacementMode,
    PlacementRegion,
    PlacementHostname,
    PlacementHost,
    PlacementUnionMember4,
    PlacementUnionMember5,
    PlacementUnionMember6,
    PlacementUnionMember7,
]


class ScriptAndVersionSettingGetResponse(BaseModel):
    annotations: Optional[Annotations] = None
    """Annotations for the Worker version.

    Annotations are not inherited across settings updates; omitting this field means
    the new version will have no annotations.
    """

    bindings: Optional[List[Binding]] = None
    """List of bindings attached to a Worker.

    You can find more about bindings on our docs:
    https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/#bindings.
    """

    compatibility_date: Optional[str] = None
    """Date indicating targeted support in the Workers runtime.

    Backwards incompatible fixes to the runtime following this date will not affect
    this Worker.
    """

    compatibility_flags: Optional[List[str]] = None
    """Flags that enable or disable certain features in the Workers runtime.

    Used to enable upcoming features or opt in or out of specific changes not
    included in a `compatibility_date`.
    """

    limits: Optional[Limits] = None
    """Limits to apply for this Worker."""

    logpush: Optional[bool] = None
    """Whether Logpush is turned on for the Worker."""

    observability: Optional[Observability] = None
    """Observability settings for the Worker."""

    placement: Optional[Placement] = None
    """
    Configuration for
    [Smart Placement](https://developers.cloudflare.com/workers/configuration/smart-placement).
    Specify mode='smart' for Smart Placement, or one of region/hostname/host.
    """

    tags: Optional[List[str]] = None
    """Tags associated with the Worker."""

    tail_consumers: Optional[List[ConsumerScript]] = None
    """List of Workers that will consume logs from the attached Worker."""

    usage_model: Optional[Literal["standard", "bundled", "unbound"]] = None
    """Usage model for the Worker invocations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "StreamCreateParams",
    "Format",
    "FormatJson",
    "FormatParquet",
    "HTTP",
    "HTTPCORS",
    "Schema",
    "SchemaField",
    "SchemaFieldInt32",
    "SchemaFieldInt64",
    "SchemaFieldFloat32",
    "SchemaFieldFloat64",
    "SchemaFieldBool",
    "SchemaFieldString",
    "SchemaFieldBinary",
    "SchemaFieldTimestamp",
    "SchemaFieldJson",
    "SchemaFieldStruct",
    "SchemaFieldList",
    "SchemaFormat",
    "SchemaFormatJson",
    "SchemaFormatParquet",
    "WorkerBinding",
]


class StreamCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Specifies the public ID of the account."""

    name: Required[str]
    """Specifies the name of the Stream."""

    format: Format

    http: HTTP

    schema: Schema

    worker_binding: WorkerBinding


class FormatJson(TypedDict, total=False):
    type: Required[Literal["json"]]

    decimal_encoding: Literal["number", "string", "bytes"]

    timestamp_format: Literal["rfc3339", "unix_millis"]

    unstructured: bool


class FormatParquet(TypedDict, total=False):
    type: Required[Literal["parquet"]]

    compression: Literal["uncompressed", "snappy", "gzip", "zstd", "lz4"]

    row_group_bytes: Optional[int]


Format: TypeAlias = Union[FormatJson, FormatParquet]


class HTTPCORS(TypedDict, total=False):
    """Specifies the CORS options for the HTTP endpoint."""

    origins: SequenceNotStr[str]


class HTTP(TypedDict, total=False):
    authentication: Required[bool]
    """Indicates that authentication is required for the HTTP endpoint."""

    enabled: Required[bool]
    """Indicates that the HTTP endpoint is enabled."""

    cors: HTTPCORS
    """Specifies the CORS options for the HTTP endpoint."""


class SchemaFieldInt32(TypedDict, total=False):
    type: Required[Literal["int32"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldInt64(TypedDict, total=False):
    type: Required[Literal["int64"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldFloat32(TypedDict, total=False):
    type: Required[Literal["float32"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldFloat64(TypedDict, total=False):
    type: Required[Literal["float64"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldBool(TypedDict, total=False):
    type: Required[Literal["bool"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldString(TypedDict, total=False):
    type: Required[Literal["string"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldBinary(TypedDict, total=False):
    type: Required[Literal["binary"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldTimestamp(TypedDict, total=False):
    type: Required[Literal["timestamp"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str

    unit: Literal["second", "millisecond", "microsecond", "nanosecond"]


class SchemaFieldJson(TypedDict, total=False):
    type: Required[Literal["json"]]

    metadata_key: Optional[str]

    name: str

    required: bool

    sql_name: str


class SchemaFieldStruct(TypedDict, total=False):
    pass


class SchemaFieldList(TypedDict, total=False):
    pass


SchemaField: TypeAlias = Union[
    SchemaFieldInt32,
    SchemaFieldInt64,
    SchemaFieldFloat32,
    SchemaFieldFloat64,
    SchemaFieldBool,
    SchemaFieldString,
    SchemaFieldBinary,
    SchemaFieldTimestamp,
    SchemaFieldJson,
    SchemaFieldStruct,
    SchemaFieldList,
]


class SchemaFormatJson(TypedDict, total=False):
    type: Required[Literal["json"]]

    decimal_encoding: Literal["number", "string", "bytes"]

    timestamp_format: Literal["rfc3339", "unix_millis"]

    unstructured: bool


class SchemaFormatParquet(TypedDict, total=False):
    type: Required[Literal["parquet"]]

    compression: Literal["uncompressed", "snappy", "gzip", "zstd", "lz4"]

    row_group_bytes: Optional[int]


SchemaFormat: TypeAlias = Union[SchemaFormatJson, SchemaFormatParquet]


class Schema(TypedDict, total=False):
    fields: Iterable[SchemaField]

    format: SchemaFormat

    inferred: Optional[bool]


class WorkerBinding(TypedDict, total=False):
    enabled: Required[bool]
    """Indicates that the worker binding is enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .subscription_configuration import SubscriptionConfiguration
from .bot_fight_mode_configuration import BotFightModeConfiguration
from .super_bot_fight_mode_likely_configuration import SuperBotFightModeLikelyConfiguration
from .super_bot_fight_mode_definitely_configuration import SuperBotFightModeDefinitelyConfiguration

__all__ = ["BotManagementGetResponse"]

BotManagementGetResponse: TypeAlias = Union[
    BotFightModeConfiguration,
    SuperBotFightModeDefinitelyConfiguration,
    SuperBotFightModeLikelyConfiguration,
    SubscriptionConfiguration,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["SilenceCreateParams", "Body"]


class SilenceCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account id"""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    end_time: str
    """When the silence ends."""

    policy_id: str
    """The unique identifier of a notification policy"""

    start_time: str
    """When the silence starts."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["CertificateRule", "Certificate"]


class Certificate(BaseModel):
    pass


class CertificateRule(BaseModel):
    """Matches any valid client certificate."""

    certificate: Certificate



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zero_trust.dlp import payload_log_update_params
from ....types.zero_trust.dlp.payload_log_get_response import PayloadLogGetResponse
from ....types.zero_trust.dlp.payload_log_update_response import PayloadLogUpdateResponse

__all__ = ["PayloadLogsResource", "AsyncPayloadLogsResource"]


class PayloadLogsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PayloadLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PayloadLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PayloadLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PayloadLogsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        masking_level: Literal["full", "partial", "clear", "default"] | Omit = omit,
        public_key: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PayloadLogUpdateResponse]:
        """Enables or disables payload logging for DLP matches.

        When enabled, matched
        content is stored for review.

        Args:
          masking_level: Masking level for payload logs.

              - `full`: The entire payload is masked.
              - `partial`: Only partial payload content is masked.
              - `clear`: No masking is applied to the payload content.
              - `default`: DLP uses its default masking behavior.

          public_key: Base64-encoded public key for encrypting payload logs.

              - Set to null or empty string to disable payload logging.
              - Set to a non-empty base64 string to enable payload logging with the given key.

              For customers with configurable payload masking feature rolled out:

              - If the field is missing, the existing setting will be kept. Note that this is
                different from setting to null or empty string.

              For all other customers:

              - If the field is missing, the existing setting will be cleared.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/dlp/payload_log", account_id=account_id),
            body=maybe_transform(
                {
                    "masking_level": masking_level,
                    "public_key": public_key,
                },
                payload_log_update_params.PayloadLogUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PayloadLogUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PayloadLogUpdateResponse]], ResultWrapper[PayloadLogUpdateResponse]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PayloadLogGetResponse]:
        """
        Gets the current payload logging configuration for DLP, showing whether matched
        content is being logged.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dlp/payload_log", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PayloadLogGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PayloadLogGetResponse]], ResultWrapper[PayloadLogGetResponse]),
        )


class AsyncPayloadLogsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPayloadLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPayloadLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPayloadLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPayloadLogsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        masking_level: Literal["full", "partial", "clear", "default"] | Omit = omit,
        public_key: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PayloadLogUpdateResponse]:
        """Enables or disables payload logging for DLP matches.

        When enabled, matched
        content is stored for review.

        Args:
          masking_level: Masking level for payload logs.

              - `full`: The entire payload is masked.
              - `partial`: Only partial payload content is masked.
              - `clear`: No masking is applied to the payload content.
              - `default`: DLP uses its default masking behavior.

          public_key: Base64-encoded public key for encrypting payload logs.

              - Set to null or empty string to disable payload logging.
              - Set to a non-empty base64 string to enable payload logging with the given key.

              For customers with configurable payload masking feature rolled out:

              - If the field is missing, the existing setting will be kept. Note that this is
                different from setting to null or empty string.

              For all other customers:

              - If the field is missing, the existing setting will be cleared.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/dlp/payload_log", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "masking_level": masking_level,
                    "public_key": public_key,
                },
                payload_log_update_params.PayloadLogUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PayloadLogUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PayloadLogUpdateResponse]], ResultWrapper[PayloadLogUpdateResponse]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PayloadLogGetResponse]:
        """
        Gets the current payload logging configuration for DLP, showing whether matched
        content is being logged.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dlp/payload_log", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PayloadLogGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PayloadLogGetResponse]], ResultWrapper[PayloadLogGetResponse]),
        )


class PayloadLogsResourceWithRawResponse:
    def __init__(self, payload_logs: PayloadLogsResource) -> None:
        self._payload_logs = payload_logs

        self.update = to_raw_response_wrapper(
            payload_logs.update,
        )
        self.get = to_raw_response_wrapper(
            payload_logs.get,
        )


class AsyncPayloadLogsResourceWithRawResponse:
    def __init__(self, payload_logs: AsyncPayloadLogsResource) -> None:
        self._payload_logs = payload_logs

        self.update = async_to_raw_response_wrapper(
            payload_logs.update,
        )
        self.get = async_to_raw_response_wrapper(
            payload_logs.get,
        )


class PayloadLogsResourceWithStreamingResponse:
    def __init__(self, payload_logs: PayloadLogsResource) -> None:
        self._payload_logs = payload_logs

        self.update = to_streamed_response_wrapper(
            payload_logs.update,
        )
        self.get = to_streamed_response_wrapper(
            payload_logs.get,
        )


class AsyncPayloadLogsResourceWithStreamingResponse:
    def __init__(self, payload_logs: AsyncPayloadLogsResource) -> None:
        self._payload_logs = payload_logs

        self.update = async_to_streamed_response_wrapper(
            payload_logs.update,
        )
        self.get = async_to_streamed_response_wrapper(
            payload_logs.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import Literal, Annotated, TypeAlias

from ...._utils import PropertyInfo
from ...._models import BaseModel

__all__ = ["SecretListResponse", "WorkersBindingKindSecretText", "WorkersBindingKindSecretKey"]


class WorkersBindingKindSecretText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_text"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindSecretKey(BaseModel):
    algorithm: object
    """Algorithm-specific key parameters.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#algorithm).
    """

    format: Literal["raw", "pkcs8", "spki", "jwk"]
    """Data format of the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#format).
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_key"]
    """The kind of resource that the binding provides."""

    usages: List[Literal["encrypt", "decrypt", "sign", "verify", "deriveKey", "deriveBits", "wrapKey", "unwrapKey"]]
    """Allowed operations with the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#keyUsages).
    """


SecretListResponse: TypeAlias = Annotated[
    Union[WorkersBindingKindSecretText, WorkersBindingKindSecretKey], PropertyInfo(discriminator="type")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .origin_post_quantum_encryption_get_response import (
    OriginPostQuantumEncryptionGetResponse as OriginPostQuantumEncryptionGetResponse,
)
from .origin_post_quantum_encryption_update_params import (
    OriginPostQuantumEncryptionUpdateParams as OriginPostQuantumEncryptionUpdateParams,
)
from .origin_post_quantum_encryption_update_response import (
    OriginPostQuantumEncryptionUpdateResponse as OriginPostQuantumEncryptionUpdateResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ai_security import (
    AISecurityResource,
    AsyncAISecurityResource,
    AISecurityResourceWithRawResponse,
    AsyncAISecurityResourceWithRawResponse,
    AISecurityResourceWithStreamingResponse,
    AsyncAISecurityResourceWithStreamingResponse,
)
from .custom_topics import (
    CustomTopicsResource,
    AsyncCustomTopicsResource,
    CustomTopicsResourceWithRawResponse,
    AsyncCustomTopicsResourceWithRawResponse,
    CustomTopicsResourceWithStreamingResponse,
    AsyncCustomTopicsResourceWithStreamingResponse,
)

__all__ = [
    "CustomTopicsResource",
    "AsyncCustomTopicsResource",
    "CustomTopicsResourceWithRawResponse",
    "AsyncCustomTopicsResourceWithRawResponse",
    "CustomTopicsResourceWithStreamingResponse",
    "AsyncCustomTopicsResourceWithStreamingResponse",
    "AISecurityResource",
    "AsyncAISecurityResource",
    "AISecurityResourceWithRawResponse",
    "AsyncAISecurityResourceWithRawResponse",
    "AISecurityResourceWithStreamingResponse",
    "AsyncAISecurityResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)
from .snippets import (
    SnippetsResource,
    AsyncSnippetsResource,
    SnippetsResourceWithRawResponse,
    AsyncSnippetsResourceWithRawResponse,
    SnippetsResourceWithStreamingResponse,
    AsyncSnippetsResourceWithStreamingResponse,
)

__all__ = [
    "ContentResource",
    "AsyncContentResource",
    "ContentResourceWithRawResponse",
    "AsyncContentResourceWithRawResponse",
    "ContentResourceWithStreamingResponse",
    "AsyncContentResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "SnippetsResource",
    "AsyncSnippetsResource",
    "SnippetsResourceWithRawResponse",
    "AsyncSnippetsResourceWithRawResponse",
    "SnippetsResourceWithStreamingResponse",
    "AsyncSnippetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["AttackMitigation"]


class AttackMitigation(BaseModel):
    """Attack mitigation settings"""

    enabled: Optional[bool] = None
    """
    When enabled, automatically mitigate random-prefix attacks to protect upstream
    DNS servers
    """

    only_when_upstream_unhealthy: Optional[bool] = None
    """Only mitigate attacks when upstream servers seem unhealthy"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "CrawlCreateParams",
    "Variant0",
    "Variant0AddScriptTag",
    "Variant0AddStyleTag",
    "Variant0Authenticate",
    "Variant0Cookie",
    "Variant0GotoOptions",
    "Variant0JsonOptions",
    "Variant0JsonOptionsCustomAI",
    "Variant0JsonOptionsResponseFormat",
    "Variant0Options",
    "Variant0Viewport",
    "Variant0WaitForSelector",
    "Variant1",
    "Variant1JsonOptions",
    "Variant1JsonOptionsCustomAI",
    "Variant1JsonOptionsResponseFormat",
    "Variant1Options",
]


class Variant0(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    url: Required[str]
    """URL to navigate to, eg. `https://example.com`."""

    cache_ttl: Annotated[float, PropertyInfo(alias="cacheTTL")]
    """Cache TTL default is 5s. Set to 0 to disable."""

    action_timeout: Annotated[float, PropertyInfo(alias="actionTimeout")]
    """
    The maximum duration allowed for the browser action to complete after the page
    has loaded (such as taking screenshots, extracting content, or generating PDFs).
    If this time limit is exceeded, the action stops and returns a timeout error.
    """

    add_script_tag: Annotated[Iterable[Variant0AddScriptTag], PropertyInfo(alias="addScriptTag")]
    """Adds a `<script>` tag into the page with the desired URL or content."""

    add_style_tag: Annotated[Iterable[Variant0AddStyleTag], PropertyInfo(alias="addStyleTag")]
    """
    Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
    `<style type="text/css">` tag with the content.
    """

    allow_request_pattern: Annotated[SequenceNotStr[str], PropertyInfo(alias="allowRequestPattern")]
    """Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'."""

    allow_resource_types: Annotated[
        List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ],
        PropertyInfo(alias="allowResourceTypes"),
    ]
    """Only allow requests that match the provided resource types, eg.

    'image' or 'script'.
    """

    authenticate: Variant0Authenticate
    """Provide credentials for HTTP authentication."""

    best_attempt: Annotated[bool, PropertyInfo(alias="bestAttempt")]
    """Attempt to proceed when 'awaited' events fail or timeout."""

    cookies: Iterable[Variant0Cookie]
    """Check [options](https://pptr.dev/api/puppeteer.page.setcookie)."""

    crawl_purposes: Annotated[List[Literal["search", "ai-input", "ai-train"]], PropertyInfo(alias="crawlPurposes")]
    """List of crawl purposes to respect Content-Signal directives in robots.txt.

    Allowed values: 'search', 'ai-input', 'ai-train'. Learn more:
    https://contentsignals.org/. Default: ['search', 'ai-input', 'ai-train'].
    """

    depth: float
    """Maximum number of levels deep the crawler will traverse from the starting URL."""

    emulate_media_type: Annotated[str, PropertyInfo(alias="emulateMediaType")]

    formats: List[Literal["html", "markdown", "json"]]
    """Formats to return. Default is `html`."""

    goto_options: Annotated[Variant0GotoOptions, PropertyInfo(alias="gotoOptions")]
    """Check [options](https://pptr.dev/api/puppeteer.gotooptions)."""

    json_options: Annotated[Variant0JsonOptions, PropertyInfo(alias="jsonOptions")]
    """Options for JSON extraction."""

    limit: float
    """Maximum number of URLs to crawl."""

    max_age: Annotated[float, PropertyInfo(alias="maxAge")]
    """Maximum age of a resource that can be returned from cache in seconds.

    Default is 1 day.
    """

    modified_since: Annotated[int, PropertyInfo(alias="modifiedSince")]
    """
    Unix timestamp (seconds since epoch) indicating to only crawl pages that were
    modified since this time. For sitemap URLs with a lastmod field, this is
    compared directly. For other URLs, the crawler will use If-Modified-Since header
    when fetching. URLs without modification information (no lastmod in sitemap and
    no Last-Modified header support) will be crawled. Note: This works in
    conjunction with maxAge - both filters must pass for a cached resource to be
    used. Must be within the last year and not in the future.
    """

    options: Variant0Options
    """Additional options for the crawler."""

    reject_request_pattern: Annotated[SequenceNotStr[str], PropertyInfo(alias="rejectRequestPattern")]
    """Block undesired requests that match the provided regex patterns, eg.

    '/^.\\**\\..(css)'.
    """

    reject_resource_types: Annotated[
        List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ],
        PropertyInfo(alias="rejectResourceTypes"),
    ]
    """Block undesired requests that match the provided resource types, eg.

    'image' or 'script'.
    """

    render: Literal[True]
    """Whether to render the page or fetch static content. True by default."""

    set_extra_http_headers: Annotated[Dict[str, str], PropertyInfo(alias="setExtraHTTPHeaders")]

    set_java_script_enabled: Annotated[bool, PropertyInfo(alias="setJavaScriptEnabled")]

    source: Literal["sitemaps", "links", "all"]
    """Source of links to crawl.

    'sitemaps' - only crawl URLs from sitemaps, 'links' - only crawl URLs scraped
    from pages, 'all' - crawl both sitemap and scraped links (default).
    """

    viewport: Variant0Viewport
    """Check [options](https://pptr.dev/api/puppeteer.page.setviewport)."""

    wait_for_selector: Annotated[Variant0WaitForSelector, PropertyInfo(alias="waitForSelector")]
    """Wait for the selector to appear in page.

    Check [options](https://pptr.dev/api/puppeteer.page.waitforselector).
    """

    wait_for_timeout: Annotated[float, PropertyInfo(alias="waitForTimeout")]
    """Waits for a specified timeout before continuing."""


class Variant0AddScriptTag(TypedDict, total=False):
    id: str

    content: str

    type: str

    url: str


class Variant0AddStyleTag(TypedDict, total=False):
    content: str

    url: str


class Variant0Authenticate(TypedDict, total=False):
    """Provide credentials for HTTP authentication."""

    password: Required[str]

    username: Required[str]


class Variant0Cookie(TypedDict, total=False):
    name: Required[str]
    """Cookie name."""

    value: Required[str]

    domain: str

    expires: float

    http_only: Annotated[bool, PropertyInfo(alias="httpOnly")]

    partition_key: Annotated[str, PropertyInfo(alias="partitionKey")]

    path: str

    priority: Literal["Low", "Medium", "High"]

    same_party: Annotated[bool, PropertyInfo(alias="sameParty")]

    same_site: Annotated[Literal["Strict", "Lax", "None"], PropertyInfo(alias="sameSite")]

    secure: bool

    source_port: Annotated[float, PropertyInfo(alias="sourcePort")]

    source_scheme: Annotated[Literal["Unset", "NonSecure", "Secure"], PropertyInfo(alias="sourceScheme")]

    url: str


class Variant0GotoOptions(TypedDict, total=False):
    """Check [options](https://pptr.dev/api/puppeteer.gotooptions)."""

    referer: str

    referrer_policy: Annotated[str, PropertyInfo(alias="referrerPolicy")]

    timeout: float

    wait_until: Annotated[
        Union[
            Literal["load", "domcontentloaded", "networkidle0", "networkidle2"],
            List[Literal["load", "domcontentloaded", "networkidle0", "networkidle2"]],
        ],
        PropertyInfo(alias="waitUntil"),
    ]


class Variant0JsonOptionsCustomAI(TypedDict, total=False):
    model: Required[str]
    """AI model to use for the request.

    Must be formed as `<provider>/<model_name>`, e.g.
    `workers-ai/@cf/meta/llama-3.3-70b-instruct-fp8-fast`.
    """

    authorization: str
    """Authorization token for the AI model: `Bearer <token>`.

    Not needed for workers-ai models.
    """


class Variant0JsonOptionsResponseFormat(TypedDict, total=False):
    type: Required[str]

    json_schema: Optional[Dict[str, Union[str, float, bool, SequenceNotStr[str], object]]]
    """Schema for the response format.

    More information here: https://developers.cloudflare.com/workers-ai/json-mode/
    """


class Variant0JsonOptions(TypedDict, total=False):
    """Options for JSON extraction."""

    custom_ai: Iterable[Variant0JsonOptionsCustomAI]
    """Optional list of custom AI models to use for the request.

    The models will be tried in the order provided, and in case a model returns an
    error, the next one will be used as fallback.
    """

    prompt: str

    response_format: Variant0JsonOptionsResponseFormat


class Variant0Options(TypedDict, total=False):
    """Additional options for the crawler."""

    exclude_patterns: Annotated[SequenceNotStr[str], PropertyInfo(alias="excludePatterns")]
    """Exclude links matching the provided wildcard patterns in the crawl job.

    Example: 'https://example.com/privacy/**'.
    """

    include_external_links: Annotated[bool, PropertyInfo(alias="includeExternalLinks")]
    """Include external links in the crawl job.

    If set to true, includeSubdomains is ignored.
    """

    include_patterns: Annotated[SequenceNotStr[str], PropertyInfo(alias="includePatterns")]
    """Include only links matching the provided wildcard patterns in the crawl job.

    Include patterns are evaluated before exclude patterns. URLs that match any of
    the specified include patterns will be included in the crawl job. Example:
    'https://example.com/blog/**'.
    """

    include_subdomains: Annotated[bool, PropertyInfo(alias="includeSubdomains")]
    """Include links to subdomains in the crawl job.

    This option is ignored if includeExternalLinks is true.
    """


class Variant0Viewport(TypedDict, total=False):
    """Check [options](https://pptr.dev/api/puppeteer.page.setviewport)."""

    height: Required[float]

    width: Required[float]

    device_scale_factor: Annotated[float, PropertyInfo(alias="deviceScaleFactor")]

    has_touch: Annotated[bool, PropertyInfo(alias="hasTouch")]

    is_landscape: Annotated[bool, PropertyInfo(alias="isLandscape")]

    is_mobile: Annotated[bool, PropertyInfo(alias="isMobile")]


class Variant0WaitForSelector(TypedDict, total=False):
    """Wait for the selector to appear in page.

    Check [options](https://pptr.dev/api/puppeteer.page.waitforselector).
    """

    selector: Required[str]

    hidden: Literal[True]

    timeout: float

    visible: Literal[True]


class Variant1(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    render: Required[Literal[False]]
    """Whether to render the page or fetch static content. True by default."""

    url: Required[str]
    """URL to navigate to, eg. `https://example.com`."""

    cache_ttl: Annotated[float, PropertyInfo(alias="cacheTTL")]
    """Cache TTL default is 5s. Set to 0 to disable."""

    crawl_purposes: Annotated[List[Literal["search", "ai-input", "ai-train"]], PropertyInfo(alias="crawlPurposes")]
    """List of crawl purposes to respect Content-Signal directives in robots.txt.

    Allowed values: 'search', 'ai-input', 'ai-train'. Learn more:
    https://contentsignals.org/. Default: ['search', 'ai-input', 'ai-train'].
    """

    depth: float
    """Maximum number of levels deep the crawler will traverse from the starting URL."""

    formats: List[Literal["html", "markdown", "json"]]
    """Formats to return. Default is `html`."""

    json_options: Annotated[Variant1JsonOptions, PropertyInfo(alias="jsonOptions")]
    """Options for JSON extraction."""

    limit: float
    """Maximum number of URLs to crawl."""

    max_age: Annotated[float, PropertyInfo(alias="maxAge")]
    """Maximum age of a resource that can be returned from cache in seconds.

    Default is 1 day.
    """

    modified_since: Annotated[int, PropertyInfo(alias="modifiedSince")]
    """
    Unix timestamp (seconds since epoch) indicating to only crawl pages that were
    modified since this time. For sitemap URLs with a lastmod field, this is
    compared directly. For other URLs, the crawler will use If-Modified-Since header
    when fetching. URLs without modification information (no lastmod in sitemap and
    no Last-Modified header support) will be crawled. Note: This works in
    conjunction with maxAge - both filters must pass for a cached resource to be
    used. Must be within the last year and not in the future.
    """

    options: Variant1Options
    """Additional options for the crawler."""

    source: Literal["sitemaps", "links", "all"]
    """Source of links to crawl.

    'sitemaps' - only crawl URLs from sitemaps, 'links' - only crawl URLs scraped
    from pages, 'all' - crawl both sitemap and scraped links (default).
    """


class Variant1JsonOptionsCustomAI(TypedDict, total=False):
    model: Required[str]
    """AI model to use for the request.

    Must be formed as `<provider>/<model_name>`, e.g.
    `workers-ai/@cf/meta/llama-3.3-70b-instruct-fp8-fast`.
    """

    authorization: str
    """Authorization token for the AI model: `Bearer <token>`.

    Not needed for workers-ai models.
    """


class Variant1JsonOptionsResponseFormat(TypedDict, total=False):
    type: Required[str]

    json_schema: Optional[Dict[str, Union[str, float, bool, SequenceNotStr[str], object]]]
    """Schema for the response format.

    More information here: https://developers.cloudflare.com/workers-ai/json-mode/
    """


class Variant1JsonOptions(TypedDict, total=False):
    """Options for JSON extraction."""

    custom_ai: Iterable[Variant1JsonOptionsCustomAI]
    """Optional list of custom AI models to use for the request.

    The models will be tried in the order provided, and in case a model returns an
    error, the next one will be used as fallback.
    """

    prompt: str

    response_format: Variant1JsonOptionsResponseFormat


class Variant1Options(TypedDict, total=False):
    """Additional options for the crawler."""

    exclude_patterns: Annotated[SequenceNotStr[str], PropertyInfo(alias="excludePatterns")]
    """Exclude links matching the provided wildcard patterns in the crawl job.

    Example: 'https://example.com/privacy/**'.
    """

    include_external_links: Annotated[bool, PropertyInfo(alias="includeExternalLinks")]
    """Include external links in the crawl job.

    If set to true, includeSubdomains is ignored.
    """

    include_patterns: Annotated[SequenceNotStr[str], PropertyInfo(alias="includePatterns")]
    """Include only links matching the provided wildcard patterns in the crawl job.

    Include patterns are evaluated before exclude patterns. URLs that match any of
    the specified include patterns will be included in the crawl job. Example:
    'https://example.com/blog/**'.
    """

    include_subdomains: Annotated[bool, PropertyInfo(alias="includeSubdomains")]
    """Include links to subdomains in the crawl job.

    This option is ignored if includeExternalLinks is true.
    """


CrawlCreateParams: TypeAlias = Union[Variant0, Variant1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["LivestreamGetLivestreamAnalyticsCompleteResponse", "Data"]


class Data(BaseModel):
    count: Optional[int] = None
    """Count of total livestreams."""

    total_ingest_seconds: Optional[int] = None
    """Total time duration for which the input was given or the meeting was streamed."""

    total_viewer_seconds: Optional[int] = None
    """Total view time for which the viewers watched the stream."""


class LivestreamGetLivestreamAnalyticsCompleteResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .sfu import (
    SFUResource,
    AsyncSFUResource,
    SFUResourceWithRawResponse,
    AsyncSFUResourceWithRawResponse,
    SFUResourceWithStreamingResponse,
    AsyncSFUResourceWithStreamingResponse,
)
from .turn import (
    TURNResource,
    AsyncTURNResource,
    TURNResourceWithRawResponse,
    AsyncTURNResourceWithRawResponse,
    TURNResourceWithStreamingResponse,
    AsyncTURNResourceWithStreamingResponse,
)
from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)

__all__ = [
    "SFUResource",
    "AsyncSFUResource",
    "SFUResourceWithRawResponse",
    "AsyncSFUResourceWithRawResponse",
    "SFUResourceWithStreamingResponse",
    "AsyncSFUResourceWithStreamingResponse",
    "TURNResource",
    "AsyncTURNResource",
    "TURNResourceWithRawResponse",
    "AsyncTURNResourceWithRawResponse",
    "TURNResourceWithStreamingResponse",
    "AsyncTURNResourceWithStreamingResponse",
    "CallsResource",
    "AsyncCallsResource",
    "CallsResourceWithRawResponse",
    "AsyncCallsResourceWithRawResponse",
    "CallsResourceWithStreamingResponse",
    "AsyncCallsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.origin_tls_client_auth import hostname_update_params
from ...types.origin_tls_client_auth.hostname_update_response import HostnameUpdateResponse
from ...types.origin_tls_client_auth.authenticated_origin_pull import AuthenticatedOriginPull

__all__ = ["HostnamesResource", "AsyncHostnamesResource"]


class HostnamesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HostnamesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return HostnamesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HostnamesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return HostnamesResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        config: Iterable[hostname_update_params.Config],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[HostnameUpdateResponse]:
        """
        Associate a hostname to a certificate and enable, disable or invalidate the
        association. If disabled, client certificate will not be sent to the hostname
        even if activated at the zone level. 100 maximum associations on a single
        certificate are allowed. Note: Use a null value for parameter _enabled_ to
        invalidate the association.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/origin_tls_client_auth/hostnames", zone_id=zone_id),
            page=SyncSinglePage[HostnameUpdateResponse],
            body=maybe_transform({"config": config}, hostname_update_params.HostnameUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=HostnameUpdateResponse,
            method="put",
        )

    def get(
        self,
        hostname: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AuthenticatedOriginPull]:
        """
        Retrieves the client certificate authentication status for a specific hostname,
        showing whether authenticated origin pulls are enabled.

        Args:
          zone_id: Identifier.

          hostname: The hostname on the origin for which the client certificate uploaded will be
              used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/origin_tls_client_auth/hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AuthenticatedOriginPull]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AuthenticatedOriginPull]], ResultWrapper[AuthenticatedOriginPull]),
        )


class AsyncHostnamesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHostnamesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHostnamesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHostnamesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncHostnamesResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        config: Iterable[hostname_update_params.Config],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[HostnameUpdateResponse, AsyncSinglePage[HostnameUpdateResponse]]:
        """
        Associate a hostname to a certificate and enable, disable or invalidate the
        association. If disabled, client certificate will not be sent to the hostname
        even if activated at the zone level. 100 maximum associations on a single
        certificate are allowed. Note: Use a null value for parameter _enabled_ to
        invalidate the association.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/origin_tls_client_auth/hostnames", zone_id=zone_id),
            page=AsyncSinglePage[HostnameUpdateResponse],
            body=maybe_transform({"config": config}, hostname_update_params.HostnameUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=HostnameUpdateResponse,
            method="put",
        )

    async def get(
        self,
        hostname: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AuthenticatedOriginPull]:
        """
        Retrieves the client certificate authentication status for a specific hostname,
        showing whether authenticated origin pulls are enabled.

        Args:
          zone_id: Identifier.

          hostname: The hostname on the origin for which the client certificate uploaded will be
              used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not hostname:
            raise ValueError(f"Expected a non-empty value for `hostname` but received {hostname!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/origin_tls_client_auth/hostnames/{hostname}", zone_id=zone_id, hostname=hostname
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AuthenticatedOriginPull]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AuthenticatedOriginPull]], ResultWrapper[AuthenticatedOriginPull]),
        )


class HostnamesResourceWithRawResponse:
    def __init__(self, hostnames: HostnamesResource) -> None:
        self._hostnames = hostnames

        self.update = to_raw_response_wrapper(
            hostnames.update,
        )
        self.get = to_raw_response_wrapper(
            hostnames.get,
        )


class AsyncHostnamesResourceWithRawResponse:
    def __init__(self, hostnames: AsyncHostnamesResource) -> None:
        self._hostnames = hostnames

        self.update = async_to_raw_response_wrapper(
            hostnames.update,
        )
        self.get = async_to_raw_response_wrapper(
            hostnames.get,
        )


class HostnamesResourceWithStreamingResponse:
    def __init__(self, hostnames: HostnamesResource) -> None:
        self._hostnames = hostnames

        self.update = to_streamed_response_wrapper(
            hostnames.update,
        )
        self.get = to_streamed_response_wrapper(
            hostnames.get,
        )


class AsyncHostnamesResourceWithStreamingResponse:
    def __init__(self, hostnames: AsyncHostnamesResource) -> None:
        self._hostnames = hostnames

        self.update = async_to_streamed_response_wrapper(
            hostnames.update,
        )
        self.get = async_to_streamed_response_wrapper(
            hostnames.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["SilenceUpdateParams", "Body"]


class SilenceUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account id"""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    id: str
    """Silence ID"""

    end_time: str
    """When the silence ends."""

    start_time: str
    """When the silence starts."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["RouteAsesParams"]


class RouteAsesParams(TypedDict, total=False):
    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    limit: int
    """Limits the number of objects returned in the response."""

    location: str
    """Filters results by location. Specify an alpha-2 location code."""

    sort_by: Annotated[
        Literal["cone", "pfxs", "ipv4", "ipv6", "rpki_valid", "rpki_invalid", "rpki_unknown"],
        PropertyInfo(alias="sortBy"),
    ]
    """Sorts results by the specified field."""

    sort_order: Annotated[Literal["ASC", "DESC"], PropertyInfo(alias="sortOrder")]
    """Sort order."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["FilterBulkDeleteResponse", "FilterBulkDeleteResponseItem"]


class FilterBulkDeleteResponseItem(BaseModel):
    id: Optional[str] = None
    """The unique identifier of the filter."""


FilterBulkDeleteResponse: TypeAlias = List[FilterBulkDeleteResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GroupListParams"]


class GroupListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier of a schema."""

    direction: Literal["asc", "desc"]
    """Defines the direction used to sort returned rule groups."""

    match: Literal["any", "all"]
    """Defines the condition for search requirements.

    When set to `all`, all the search requirements must match. When set to `any`,
    only one of the search requirements has to match.
    """

    mode: Literal["on", "off"]
    """Defines the state of the rules contained in the rule group.

    When `on`, the rules in the group are configurable/usable.
    """

    name: str
    """Defines the name of the rule group."""

    order: Literal["mode", "rules_count"]
    """Defines the field used to sort returned rule groups."""

    page: float
    """Defines the page number of paginated results."""

    per_page: float
    """Defines the number of rule groups per page."""

    rules_count: float
    """Defines the number of rules in the current rule group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.radar.bgp.rpki import aspa_changes_params, aspa_snapshot_params, aspa_timeseries_params
from .....types.radar.bgp.rpki.aspa_changes_response import ASPAChangesResponse
from .....types.radar.bgp.rpki.aspa_snapshot_response import ASPASnapshotResponse
from .....types.radar.bgp.rpki.aspa_timeseries_response import ASPATimeseriesResponse

__all__ = ["ASPAResource", "AsyncASPAResource"]


class ASPAResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ASPAResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ASPAResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ASPAResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ASPAResourceWithStreamingResponse(self)

    def changes(
        self,
        *,
        asn: int | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        include_asn_info: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ASPAChangesResponse:
        """
        Retrieves ASPA (Autonomous System Provider Authorization) changes over time.
        Returns daily aggregated changes including additions, removals, and
        modifications of ASPA objects.

        Args:
          asn: Filter changes involving this ASN (as customer or provider).

          date_end: End of the date range (inclusive).

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          include_asn_info: Include ASN metadata (name, country) in response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/rpki/aspa/changes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_start": date_start,
                        "format": format,
                        "include_asn_info": include_asn_info,
                    },
                    aspa_changes_params.ASPAChangesParams,
                ),
                post_parser=ResultWrapper[ASPAChangesResponse]._unwrapper,
            ),
            cast_to=cast(Type[ASPAChangesResponse], ResultWrapper[ASPAChangesResponse]),
        )

    def snapshot(
        self,
        *,
        customer_asn: int | Omit = omit,
        date: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        include_asn_info: bool | Omit = omit,
        provider_asn: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ASPASnapshotResponse:
        """
        Retrieves current or historical ASPA (Autonomous System Provider Authorization)
        objects. ASPA objects define which ASNs are authorized upstream providers for a
        customer ASN.

        Args:
          customer_asn: Filter by customer ASN (the ASN publishing the ASPA object).

          date: Filters results by the specified datetime (ISO 8601).

          format: Format in which results will be returned.

          include_asn_info: Include ASN metadata (name, country) in response.

          provider_asn: Filter by provider ASN (an authorized upstream provider in ASPA objects).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/rpki/aspa/snapshot",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_asn": customer_asn,
                        "date": date,
                        "format": format,
                        "include_asn_info": include_asn_info,
                        "provider_asn": provider_asn,
                    },
                    aspa_snapshot_params.ASPASnapshotParams,
                ),
                post_parser=ResultWrapper[ASPASnapshotResponse]._unwrapper,
            ),
            cast_to=cast(Type[ASPASnapshotResponse], ResultWrapper[ASPASnapshotResponse]),
        )

    def timeseries(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        rir: List[Literal["RIPE_NCC", "ARIN", "APNIC", "LACNIC", "AFRINIC"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ASPATimeseriesResponse:
        """Retrieves ASPA (Autonomous System Provider Authorization) object count over
        time.

        Supports filtering by RIR or location (country code) to generate multiple
        named series. If no RIR or location filter is specified, returns total count.

        Args:
          date_end: End of the date range (inclusive).

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 location
              codes.

          name: Array of names used to label the series in the response.

          rir: Filter by Regional Internet Registry (RIR). Multiple RIRs generate multiple
              series.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bgp/rpki/aspa/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "name": name,
                        "rir": rir,
                    },
                    aspa_timeseries_params.ASPATimeseriesParams,
                ),
                post_parser=ResultWrapper[ASPATimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[ASPATimeseriesResponse], ResultWrapper[ASPATimeseriesResponse]),
        )


class AsyncASPAResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncASPAResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncASPAResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncASPAResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncASPAResourceWithStreamingResponse(self)

    async def changes(
        self,
        *,
        asn: int | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        include_asn_info: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ASPAChangesResponse:
        """
        Retrieves ASPA (Autonomous System Provider Authorization) changes over time.
        Returns daily aggregated changes including additions, removals, and
        modifications of ASPA objects.

        Args:
          asn: Filter changes involving this ASN (as customer or provider).

          date_end: End of the date range (inclusive).

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          include_asn_info: Include ASN metadata (name, country) in response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/rpki/aspa/changes",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "date_end": date_end,
                        "date_start": date_start,
                        "format": format,
                        "include_asn_info": include_asn_info,
                    },
                    aspa_changes_params.ASPAChangesParams,
                ),
                post_parser=ResultWrapper[ASPAChangesResponse]._unwrapper,
            ),
            cast_to=cast(Type[ASPAChangesResponse], ResultWrapper[ASPAChangesResponse]),
        )

    async def snapshot(
        self,
        *,
        customer_asn: int | Omit = omit,
        date: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        include_asn_info: bool | Omit = omit,
        provider_asn: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ASPASnapshotResponse:
        """
        Retrieves current or historical ASPA (Autonomous System Provider Authorization)
        objects. ASPA objects define which ASNs are authorized upstream providers for a
        customer ASN.

        Args:
          customer_asn: Filter by customer ASN (the ASN publishing the ASPA object).

          date: Filters results by the specified datetime (ISO 8601).

          format: Format in which results will be returned.

          include_asn_info: Include ASN metadata (name, country) in response.

          provider_asn: Filter by provider ASN (an authorized upstream provider in ASPA objects).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/rpki/aspa/snapshot",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "customer_asn": customer_asn,
                        "date": date,
                        "format": format,
                        "include_asn_info": include_asn_info,
                        "provider_asn": provider_asn,
                    },
                    aspa_snapshot_params.ASPASnapshotParams,
                ),
                post_parser=ResultWrapper[ASPASnapshotResponse]._unwrapper,
            ),
            cast_to=cast(Type[ASPASnapshotResponse], ResultWrapper[ASPASnapshotResponse]),
        )

    async def timeseries(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        rir: List[Literal["RIPE_NCC", "ARIN", "APNIC", "LACNIC", "AFRINIC"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ASPATimeseriesResponse:
        """Retrieves ASPA (Autonomous System Provider Authorization) object count over
        time.

        Supports filtering by RIR or location (country code) to generate multiple
        named series. If no RIR or location filter is specified, returns total count.

        Args:
          date_end: End of the date range (inclusive).

          date_start: Start of the date range (inclusive).

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 location
              codes.

          name: Array of names used to label the series in the response.

          rir: Filter by Regional Internet Registry (RIR). Multiple RIRs generate multiple
              series.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bgp/rpki/aspa/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date_end": date_end,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "name": name,
                        "rir": rir,
                    },
                    aspa_timeseries_params.ASPATimeseriesParams,
                ),
                post_parser=ResultWrapper[ASPATimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[ASPATimeseriesResponse], ResultWrapper[ASPATimeseriesResponse]),
        )


class ASPAResourceWithRawResponse:
    def __init__(self, aspa: ASPAResource) -> None:
        self._aspa = aspa

        self.changes = to_raw_response_wrapper(
            aspa.changes,
        )
        self.snapshot = to_raw_response_wrapper(
            aspa.snapshot,
        )
        self.timeseries = to_raw_response_wrapper(
            aspa.timeseries,
        )


class AsyncASPAResourceWithRawResponse:
    def __init__(self, aspa: AsyncASPAResource) -> None:
        self._aspa = aspa

        self.changes = async_to_raw_response_wrapper(
            aspa.changes,
        )
        self.snapshot = async_to_raw_response_wrapper(
            aspa.snapshot,
        )
        self.timeseries = async_to_raw_response_wrapper(
            aspa.timeseries,
        )


class ASPAResourceWithStreamingResponse:
    def __init__(self, aspa: ASPAResource) -> None:
        self._aspa = aspa

        self.changes = to_streamed_response_wrapper(
            aspa.changes,
        )
        self.snapshot = to_streamed_response_wrapper(
            aspa.snapshot,
        )
        self.timeseries = to_streamed_response_wrapper(
            aspa.timeseries,
        )


class AsyncASPAResourceWithStreamingResponse:
    def __init__(self, aspa: AsyncASPAResource) -> None:
        self._aspa = aspa

        self.changes = async_to_streamed_response_wrapper(
            aspa.changes,
        )
        self.snapshot = async_to_streamed_response_wrapper(
            aspa.snapshot,
        )
        self.timeseries = async_to_streamed_response_wrapper(
            aspa.timeseries,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .asn import ASN as ASN
from .role import Role as Role
from .token import Token as Token
from .member import Member as Member
from .result import Result as Result
from .audit_log import AuditLog as AuditLog
from .rate_plan import RatePlan as RatePlan
from .error_data import ErrorData as ErrorData
from .identifier import Identifier as Identifier
from .permission import Permission as Permission
from .token_value import TokenValue as TokenValue
from .subscription import Subscription as Subscription
from .token_policy import TokenPolicy as TokenPolicy
from .response_info import ResponseInfo as ResponseInfo
from .certificate_ca import CertificateCA as CertificateCA
from .sort_direction import SortDirection as SortDirection
from .pagination_info import PaginationInfo as PaginationInfo
from .permission_grant import PermissionGrant as PermissionGrant
from .cloudflare_tunnel import CloudflareTunnel as CloudflareTunnel
from .subscription_zone import SubscriptionZone as SubscriptionZone
from .load_balancer_preview import LoadBalancerPreview as LoadBalancerPreview
from .subscription_component import SubscriptionComponent as SubscriptionComponent
from .certificate_request_type import CertificateRequestType as CertificateRequestType
from .token_condition_cidr_list import TokenConditionCIDRList as TokenConditionCIDRList



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .resources import (
    ResourcesResource,
    AsyncResourcesResource,
    ResourcesResourceWithRawResponse,
    AsyncResourcesResourceWithRawResponse,
    ResourcesResourceWithStreamingResponse,
    AsyncResourcesResourceWithStreamingResponse,
)
from .recipients import (
    RecipientsResource,
    AsyncRecipientsResource,
    RecipientsResourceWithRawResponse,
    AsyncRecipientsResourceWithRawResponse,
    RecipientsResourceWithStreamingResponse,
    AsyncRecipientsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.resource_sharing import (
    resource_sharing_get_params,
    resource_sharing_list_params,
    resource_sharing_create_params,
    resource_sharing_update_params,
)
from ...types.resource_sharing.resource_sharing_get_response import ResourceSharingGetResponse
from ...types.resource_sharing.resource_sharing_list_response import ResourceSharingListResponse
from ...types.resource_sharing.resource_sharing_create_response import ResourceSharingCreateResponse
from ...types.resource_sharing.resource_sharing_delete_response import ResourceSharingDeleteResponse
from ...types.resource_sharing.resource_sharing_update_response import ResourceSharingUpdateResponse

__all__ = ["ResourceSharingResource", "AsyncResourceSharingResource"]


class ResourceSharingResource(SyncAPIResource):
    @cached_property
    def recipients(self) -> RecipientsResource:
        return RecipientsResource(self._client)

    @cached_property
    def resources(self) -> ResourcesResource:
        return ResourcesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ResourceSharingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ResourceSharingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResourceSharingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ResourceSharingResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        recipients: Iterable[resource_sharing_create_params.Recipient],
        resources: Iterable[resource_sharing_create_params.Resource],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingCreateResponse]:
        """
        Creates a new resource share for sharing Cloudflare resources with other
        accounts or organizations.

        Args:
          account_id: Account identifier.

          name: The name of the share.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/shares", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "recipients": recipients,
                    "resources": resources,
                },
                resource_sharing_create_params.ResourceSharingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceSharingCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingCreateResponse]], ResultWrapper[ResourceSharingCreateResponse]),
        )

    def update(
        self,
        share_id: str,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingUpdateResponse]:
        """
        Updating is not immediate, an updated share object with a new status will be
        returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          name: The name of the share.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/shares/{share_id}", account_id=account_id, share_id=share_id),
            body=maybe_transform({"name": name}, resource_sharing_update_params.ResourceSharingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceSharingUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingUpdateResponse]], ResultWrapper[ResourceSharingUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        include_recipient_counts: bool | Omit = omit,
        include_resources: bool | Omit = omit,
        kind: Literal["sent", "received"] | Omit = omit,
        order: Literal["name", "created"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        resource_types: List[
            Literal[
                "custom-ruleset",
                "gateway-policy",
                "gateway-destination-ip",
                "gateway-block-page-settings",
                "gateway-extended-email-matching",
                "idp-federation-grant",
            ]
        ]
        | Omit = omit,
        status: Literal["active", "deleting", "deleted"] | Omit = omit,
        tag: SequenceNotStr[str] | Omit = omit,
        target_type: Literal["account", "organization"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ResourceSharingListResponse]:
        """
        Lists all account shares.

        Args:
          account_id: Account identifier.

          direction: Direction to sort objects.

          include_recipient_counts: Include recipient counts in the response.

          include_resources: Include resources in the response.

          kind: Filter shares by kind.

          order: Order shares by values in the given field.

          page: Page number.

          per_page: Number of objects to return per page.

          resource_types: Filter share resources by resource_types.

          status: Filter shares by status.

          tag: Filter shares by tag. Each value is either `key=value` (matches shares whose
              tags contain that key/value pair) or `key` alone (matches shares that have any
              value for that key). May be repeated; multiple `tag` parameters are ANDed
              together. Maximum 20 `tag` parameters per request.

          target_type: Filter shares by target_type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/shares", account_id=account_id),
            page=SyncV4PagePaginationArray[ResourceSharingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "include_recipient_counts": include_recipient_counts,
                        "include_resources": include_resources,
                        "kind": kind,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "resource_types": resource_types,
                        "status": status,
                        "tag": tag,
                        "target_type": target_type,
                    },
                    resource_sharing_list_params.ResourceSharingListParams,
                ),
            ),
            model=ResourceSharingListResponse,
        )

    def delete(
        self,
        share_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingDeleteResponse]:
        """
        Deletion is not immediate, an updated share object with a new status will be
        returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/shares/{share_id}", account_id=account_id, share_id=share_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceSharingDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingDeleteResponse]], ResultWrapper[ResourceSharingDeleteResponse]),
        )

    def get(
        self,
        share_id: str,
        *,
        account_id: str,
        include_recipient_counts: bool | Omit = omit,
        include_resources: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingGetResponse]:
        """
        Fetches share by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          include_recipient_counts: Include recipient counts in the response.

          include_resources: Include resources in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/shares/{share_id}", account_id=account_id, share_id=share_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_recipient_counts": include_recipient_counts,
                        "include_resources": include_resources,
                    },
                    resource_sharing_get_params.ResourceSharingGetParams,
                ),
                post_parser=ResultWrapper[Optional[ResourceSharingGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingGetResponse]], ResultWrapper[ResourceSharingGetResponse]),
        )


class AsyncResourceSharingResource(AsyncAPIResource):
    @cached_property
    def recipients(self) -> AsyncRecipientsResource:
        return AsyncRecipientsResource(self._client)

    @cached_property
    def resources(self) -> AsyncResourcesResource:
        return AsyncResourcesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncResourceSharingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResourceSharingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResourceSharingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncResourceSharingResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        recipients: Iterable[resource_sharing_create_params.Recipient],
        resources: Iterable[resource_sharing_create_params.Resource],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingCreateResponse]:
        """
        Creates a new resource share for sharing Cloudflare resources with other
        accounts or organizations.

        Args:
          account_id: Account identifier.

          name: The name of the share.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/shares", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "recipients": recipients,
                    "resources": resources,
                },
                resource_sharing_create_params.ResourceSharingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceSharingCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingCreateResponse]], ResultWrapper[ResourceSharingCreateResponse]),
        )

    async def update(
        self,
        share_id: str,
        *,
        account_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingUpdateResponse]:
        """
        Updating is not immediate, an updated share object with a new status will be
        returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          name: The name of the share.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/shares/{share_id}", account_id=account_id, share_id=share_id),
            body=await async_maybe_transform(
                {"name": name}, resource_sharing_update_params.ResourceSharingUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceSharingUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingUpdateResponse]], ResultWrapper[ResourceSharingUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        include_recipient_counts: bool | Omit = omit,
        include_resources: bool | Omit = omit,
        kind: Literal["sent", "received"] | Omit = omit,
        order: Literal["name", "created"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        resource_types: List[
            Literal[
                "custom-ruleset",
                "gateway-policy",
                "gateway-destination-ip",
                "gateway-block-page-settings",
                "gateway-extended-email-matching",
                "idp-federation-grant",
            ]
        ]
        | Omit = omit,
        status: Literal["active", "deleting", "deleted"] | Omit = omit,
        tag: SequenceNotStr[str] | Omit = omit,
        target_type: Literal["account", "organization"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ResourceSharingListResponse, AsyncV4PagePaginationArray[ResourceSharingListResponse]]:
        """
        Lists all account shares.

        Args:
          account_id: Account identifier.

          direction: Direction to sort objects.

          include_recipient_counts: Include recipient counts in the response.

          include_resources: Include resources in the response.

          kind: Filter shares by kind.

          order: Order shares by values in the given field.

          page: Page number.

          per_page: Number of objects to return per page.

          resource_types: Filter share resources by resource_types.

          status: Filter shares by status.

          tag: Filter shares by tag. Each value is either `key=value` (matches shares whose
              tags contain that key/value pair) or `key` alone (matches shares that have any
              value for that key). May be repeated; multiple `tag` parameters are ANDed
              together. Maximum 20 `tag` parameters per request.

          target_type: Filter shares by target_type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/shares", account_id=account_id),
            page=AsyncV4PagePaginationArray[ResourceSharingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "include_recipient_counts": include_recipient_counts,
                        "include_resources": include_resources,
                        "kind": kind,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "resource_types": resource_types,
                        "status": status,
                        "tag": tag,
                        "target_type": target_type,
                    },
                    resource_sharing_list_params.ResourceSharingListParams,
                ),
            ),
            model=ResourceSharingListResponse,
        )

    async def delete(
        self,
        share_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingDeleteResponse]:
        """
        Deletion is not immediate, an updated share object with a new status will be
        returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/shares/{share_id}", account_id=account_id, share_id=share_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceSharingDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingDeleteResponse]], ResultWrapper[ResourceSharingDeleteResponse]),
        )

    async def get(
        self,
        share_id: str,
        *,
        account_id: str,
        include_recipient_counts: bool | Omit = omit,
        include_resources: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceSharingGetResponse]:
        """
        Fetches share by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          include_recipient_counts: Include recipient counts in the response.

          include_resources: Include resources in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/shares/{share_id}", account_id=account_id, share_id=share_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_recipient_counts": include_recipient_counts,
                        "include_resources": include_resources,
                    },
                    resource_sharing_get_params.ResourceSharingGetParams,
                ),
                post_parser=ResultWrapper[Optional[ResourceSharingGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceSharingGetResponse]], ResultWrapper[ResourceSharingGetResponse]),
        )


class ResourceSharingResourceWithRawResponse:
    def __init__(self, resource_sharing: ResourceSharingResource) -> None:
        self._resource_sharing = resource_sharing

        self.create = to_raw_response_wrapper(
            resource_sharing.create,
        )
        self.update = to_raw_response_wrapper(
            resource_sharing.update,
        )
        self.list = to_raw_response_wrapper(
            resource_sharing.list,
        )
        self.delete = to_raw_response_wrapper(
            resource_sharing.delete,
        )
        self.get = to_raw_response_wrapper(
            resource_sharing.get,
        )

    @cached_property
    def recipients(self) -> RecipientsResourceWithRawResponse:
        return RecipientsResourceWithRawResponse(self._resource_sharing.recipients)

    @cached_property
    def resources(self) -> ResourcesResourceWithRawResponse:
        return ResourcesResourceWithRawResponse(self._resource_sharing.resources)


class AsyncResourceSharingResourceWithRawResponse:
    def __init__(self, resource_sharing: AsyncResourceSharingResource) -> None:
        self._resource_sharing = resource_sharing

        self.create = async_to_raw_response_wrapper(
            resource_sharing.create,
        )
        self.update = async_to_raw_response_wrapper(
            resource_sharing.update,
        )
        self.list = async_to_raw_response_wrapper(
            resource_sharing.list,
        )
        self.delete = async_to_raw_response_wrapper(
            resource_sharing.delete,
        )
        self.get = async_to_raw_response_wrapper(
            resource_sharing.get,
        )

    @cached_property
    def recipients(self) -> AsyncRecipientsResourceWithRawResponse:
        return AsyncRecipientsResourceWithRawResponse(self._resource_sharing.recipients)

    @cached_property
    def resources(self) -> AsyncResourcesResourceWithRawResponse:
        return AsyncResourcesResourceWithRawResponse(self._resource_sharing.resources)


class ResourceSharingResourceWithStreamingResponse:
    def __init__(self, resource_sharing: ResourceSharingResource) -> None:
        self._resource_sharing = resource_sharing

        self.create = to_streamed_response_wrapper(
            resource_sharing.create,
        )
        self.update = to_streamed_response_wrapper(
            resource_sharing.update,
        )
        self.list = to_streamed_response_wrapper(
            resource_sharing.list,
        )
        self.delete = to_streamed_response_wrapper(
            resource_sharing.delete,
        )
        self.get = to_streamed_response_wrapper(
            resource_sharing.get,
        )

    @cached_property
    def recipients(self) -> RecipientsResourceWithStreamingResponse:
        return RecipientsResourceWithStreamingResponse(self._resource_sharing.recipients)

    @cached_property
    def resources(self) -> ResourcesResourceWithStreamingResponse:
        return ResourcesResourceWithStreamingResponse(self._resource_sharing.resources)


class AsyncResourceSharingResourceWithStreamingResponse:
    def __init__(self, resource_sharing: AsyncResourceSharingResource) -> None:
        self._resource_sharing = resource_sharing

        self.create = async_to_streamed_response_wrapper(
            resource_sharing.create,
        )
        self.update = async_to_streamed_response_wrapper(
            resource_sharing.update,
        )
        self.list = async_to_streamed_response_wrapper(
            resource_sharing.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            resource_sharing.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            resource_sharing.get,
        )

    @cached_property
    def recipients(self) -> AsyncRecipientsResourceWithStreamingResponse:
        return AsyncRecipientsResourceWithStreamingResponse(self._resource_sharing.recipients)

    @cached_property
    def resources(self) -> AsyncResourcesResourceWithStreamingResponse:
        return AsyncResourcesResourceWithStreamingResponse(self._resource_sharing.resources)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["CertificateRequestType"]

CertificateRequestType: TypeAlias = Literal["origin-rsa", "origin-ecc", "keyless-certificate"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["IndexQueryParams"]


class IndexQueryParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    vector: Required[Iterable[float]]
    """The search vector that will be used to find the nearest neighbors."""

    filter: object
    """A metadata filter expression used to limit nearest neighbor results."""

    return_metadata: Annotated[Literal["none", "indexed", "all"], PropertyInfo(alias="returnMetadata")]
    """
    Whether to return no metadata, indexed metadata or all metadata associated with
    the closest vectors.
    """

    return_values: Annotated[bool, PropertyInfo(alias="returnValues")]
    """Whether to return the values associated with the closest vectors."""

    top_k: Annotated[float, PropertyInfo(alias="topK")]
    """The number of nearest neighbors to find."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["ClientCertificateInput"]


class ClientCertificateInput(BaseModel):
    certificate_id: str
    """UUID of Cloudflare managed certificate."""

    cn: str
    """Common Name that is protected by the certificate."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .entries import (
    EntriesResource,
    AsyncEntriesResource,
    EntriesResourceWithRawResponse,
    AsyncEntriesResourceWithRawResponse,
    EntriesResourceWithStreamingResponse,
    AsyncEntriesResourceWithStreamingResponse,
)
from .content_lists import (
    ContentListsResource,
    AsyncContentListsResource,
    ContentListsResourceWithRawResponse,
    AsyncContentListsResourceWithRawResponse,
    ContentListsResourceWithStreamingResponse,
    AsyncContentListsResourceWithStreamingResponse,
)

__all__ = [
    "EntriesResource",
    "AsyncEntriesResource",
    "EntriesResourceWithRawResponse",
    "AsyncEntriesResourceWithRawResponse",
    "EntriesResourceWithStreamingResponse",
    "AsyncEntriesResourceWithStreamingResponse",
    "ContentListsResource",
    "AsyncContentListsResource",
    "ContentListsResourceWithRawResponse",
    "AsyncContentListsResourceWithRawResponse",
    "ContentListsResourceWithStreamingResponse",
    "AsyncContentListsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PageShieldUpdateParams"]


class PageShieldUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier"""

    enabled: bool
    """When true, indicates that Page Shield is enabled."""

    use_cloudflare_reporting_endpoint: bool
    """
    When true, CSP reports will be sent to
    https://csp-reporting.cloudflare.com/cdn-cgi/script_monitor/report
    """

    use_connection_url_path: bool
    """When true, the paths associated with connections URLs will also be analyzed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ....._models import BaseModel
from ...percentiles import Percentiles

__all__ = ["HTTPDetailsPercentiles"]


class HTTPDetailsPercentiles(BaseModel):
    dns_response_time_ms: Optional[Percentiles] = FieldInfo(alias="dnsResponseTimeMs", default=None)

    resource_fetch_time_ms: Optional[Percentiles] = FieldInfo(alias="resourceFetchTimeMs", default=None)

    server_response_time_ms: Optional[Percentiles] = FieldInfo(alias="serverResponseTimeMs", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["LocationListResponse", "Location"]


class Location(BaseModel):
    alpha2: str

    continent: str

    latitude: str
    """A numeric string."""

    longitude: str
    """A numeric string."""

    name: str

    region: str

    subregion: str


class LocationListResponse(BaseModel):
    locations: List[Location]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ResourceGroupListParams"]


class ResourceGroupListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    id: str
    """ID of the resource group to be fetched."""

    name: str
    """Name of the resource group to be fetched."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SettingUpdateParams"]


class SettingUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    search_engine_crawler_bypass: bool
    """
    Whether to allow verified search engine crawlers to bypass all waiting rooms on
    this zone. Verified search engine crawlers will not be tracked or counted by the
    waiting room system, and will not appear in waiting room analytics.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["RoutingTimeseriesGroupsV2Params"]


class RoutingTimeseriesGroupsV2Params(TypedDict, total=False):
    agg_interval: Annotated[Literal["15m", "1h", "1d", "1w"], PropertyInfo(alias="aggInterval")]
    """Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).

    Refer to
    [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).
    """

    arc: List[Literal["PASS", "NONE", "FAIL"]]
    """Filters results by ARC (Authenticated Received Chain) validation."""

    date_end: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_range: Annotated[SequenceNotStr[str], PropertyInfo(alias="dateRange")]
    """Filters results by date range.

    For example, use `7d` and `7dcontrol` to compare this week with the previous
    week. Use this parameter or set specific start and end dates (`dateStart` and
    `dateEnd` parameters).
    """

    date_start: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range."""

    dkim: List[Literal["PASS", "NONE", "FAIL"]]
    """Filters results by DKIM (DomainKeys Identified Mail) validation status."""

    dmarc: List[Literal["PASS", "NONE", "FAIL"]]
    """
    Filters results by DMARC (Domain-based Message Authentication, Reporting and
    Conformance) validation status.
    """

    encrypted: List[Literal["ENCRYPTED", "NOT_ENCRYPTED"]]
    """Filters results by encryption status (encrypted vs. not-encrypted)."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    ip_version: Annotated[List[Literal["IPv4", "IPv6"]], PropertyInfo(alias="ipVersion")]
    """Filters results by IP version (Ipv4 vs. IPv6)."""

    limit_per_group: Annotated[int, PropertyInfo(alias="limitPerGroup")]
    """
    Limits the number of objects per group to the top items within the specified
    time range. When item count exceeds the limit, extra items appear grouped under
    an "other" category.
    """

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""

    spf: List[Literal["PASS", "NONE", "FAIL"]]
    """Filters results by SPF (Sender Policy Framework) validation status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["SubdomainDeleteResponse"]


class SubdomainDeleteResponse(BaseModel):
    enabled: bool
    """Whether the Worker is available on the workers.dev subdomain."""

    previews_enabled: bool
    """Whether the Worker's Preview URLs are available on the workers.dev subdomain."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DomainUpdateParams"]


class DomainUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    hostname: Required[str]
    """Hostname of the domain.

    Can be either the zone apex or a subdomain of the zone. Requests to this
    hostname will be routed to the configured Worker.
    """

    service: Required[str]
    """Name of the Worker associated with the domain.

    Requests to the configured hostname will be routed to this Worker.
    """

    environment: str
    """Worker environment associated with the domain."""

    zone_id: str
    """ID of the zone containing the domain hostname."""

    zone_name: str
    """Name of the zone containing the domain hostname."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["CertificateCA"]

CertificateCA: TypeAlias = Literal["digicert", "google", "lets_encrypt", "ssl_com"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["Dataset", "Column", "Upload"]


class Column(BaseModel):
    entry_id: str

    header_name: str

    num_cells: int

    upload_status: Literal["empty", "uploading", "pending", "processing", "failed", "complete"]


class Upload(BaseModel):
    num_cells: int

    status: Literal["empty", "uploading", "pending", "processing", "failed", "complete"]

    version: int


class Dataset(BaseModel):
    id: str

    columns: List[Column]

    created_at: datetime

    encoding_version: int

    name: str

    num_cells: int

    secret: bool

    status: Literal["empty", "uploading", "pending", "processing", "failed", "complete"]

    updated_at: datetime
    """Stores when the dataset was last updated.

    This includes name or description changes as well as uploads.
    """

    uploads: List[Upload]

    case_sensitive: Optional[bool] = None

    description: Optional[str] = None
    """The description of the dataset."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = [
    "NamespaceBulkGetResponse",
    "WorkersKVBulkGetResult",
    "WorkersKVBulkGetResultWithMetadata",
    "WorkersKVBulkGetResultWithMetadataValues",
]


class WorkersKVBulkGetResult(BaseModel):
    values: Optional[Dict[str, Union[str, float, bool, Dict[str, object]]]] = None
    """Requested keys are paired with their values in an object."""


class WorkersKVBulkGetResultWithMetadataValues(BaseModel):
    metadata: object
    """The metadata associated with the key."""

    value: object
    """The value associated with the key."""

    expiration: Optional[float] = None
    """
    Expires the key at a certain time, measured in number of seconds since the UNIX
    epoch.
    """


class WorkersKVBulkGetResultWithMetadata(BaseModel):
    values: Optional[Dict[str, Optional[WorkersKVBulkGetResultWithMetadataValues]]] = None
    """Requested keys are paired with their values and metadata in an object."""


NamespaceBulkGetResponse: TypeAlias = Union[WorkersKVBulkGetResult, WorkersKVBulkGetResultWithMetadata, None]
